
//-----------------------------------------------------
// Solution of Boundary Value Problem with the shooting method.
// Equation to solve:
// x''+lambda*x=0 
// the interval [0,1] 
// Boundary conditions are: x(0)=x(1)=0, x'(0)=1.
//---------------------------------------------------
// compile with 
// gcc -o EigenShooting EigenShooting.c  -L/usr/local/dislin -ldislnc -lm -lX11

#include "/usr/local/dislin/dislin.h"
#include <math.h>
#include <stdio.h>

#define PI 3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679
#define max 2000

int max_steps = 1.;   // t in [0, max_steps]
int i, j, k, nn;     // Loop counter    
int N=450; // amount of Newton's steps
    float T[max];    
    float X[max];              
    float Y[max];
    float Z[max];
    float F1[max];
    float F2[max];
    float So[max];
    float Sopt[3];
  //  float St[max];
   float s0[3]={0.5, 50.0, 100.0};
double h = 0.005; // time step
// initial conditions for the corresponding i.v.p.
double x = 0.0; // boundary condition on the left side of the interval [0, 1]
double y = 1.0; // x'(0)=1;
double s = 0.5; // initial slope 
double b= 0.0; // boundary condition on the right side of the interval [0, 1]
double ds=0.01;

double k1x, k2x, k3x, k4x, k1y, k2y, k3y, k4y, k1z, k2z, k3z, k4z;  /* Function values for RK4    */
double t=0.0;

// r.h.s. for the o.d.e's of the 1st order
double funct_x(double t, double x, double y, double z) {
  return y;
}

double funct_y(double t, double x, double y, double z) {
  return -z*x;
}
double funct_z(double t, double x, double y, double z) {
  return 0.0;
}

// RK4 method
void rk43(double h, double max_steps, double t, double x, double y, double z)
{
    T[0] = t;
    X[0] = x;
    Y[0] = y;
    Z[0] = z;
    j    = 0;
while(T[j] < max_steps)
    {
      if( (T[j] + h) > max_steps ) h = max_steps - T[j];   /* Calculation of the last step */
      t  = T[j];
      x  = X[j];
      y  = Y[j];
      z  = Z[j];

    k1x = h * funct_x(t, x, y, z);
    k1y = h * funct_y(t, x, y, z);
    k1z = h * funct_z(t, x, y, z);
    
    k2x = h * funct_x(t+0.5*h, x + 0.5 * k1x, y + 0.5 * k1y, z+0.5*k1z);
    k2y = h * funct_y(t+0.5*h, x + 0.5 * k1x, y + 0.5 * k1y, z+0.5*k1z);
    k2z = h * funct_z(t+0.5*h, x + 0.5 * k1x, y + 0.5 * k1y, z+0.5*k1z);
    

    k3x = h * funct_x(t+0.5*h, x + 0.5 * k2x, y + 0.5 * k2y, z+0.5*k2z);
    k3y = h * funct_y(t+0.5*h, x + 0.5 * k2x, y + 0.5 * k2y, z+0.5*k2z);
    k3z = h * funct_z(t+0.5*h, x + 0.5 * k2x, y + 0.5 * k2y, z+0.5*k2z);

    k4x = h * funct_x(t+h, x + k3x, y + k3y, z+0.5*k3z);
    k4y = h * funct_y(t+h, x + k3x, y + k3y, z+0.5*k3z);
    k4z = h * funct_z(t+h, x + k3x, y + k3y, z+0.5*k3z);

    X[j+1]= x+(k1x + 2.0 * k2x + 2.0 * k3x + k4x) / 6.0;

    Y[j+1]= y+(k1y + 2.0 * k2y + 2.0 * k3y + k4y) / 6.0;

    Z[j+1]= z+(k1z + 2.0 * k2z + 2.0 * k3z + k4z) / 6.0;

    T[j+1] = t + h;
    j++;
   }
}
// Newton's method

//--------------------------------------------------------------------
main ()
{

//Newton's method
for (k=0; k<=2;){
So[0]=s0[k];
for (i=0; i<=N; i++){
rk43(h,max_steps,t,x,y,So[i]);
nn=j;
F1[i]=X[nn]-b;
rk43(h,max_steps,t,x,y,So[i]+ds);
nn=j;
F2[i]=X[nn]-b;
So[i+1]=So[i]-(F1[i]*ds)/(F2[i]-F1[i]);
}
Sopt[k]=So[i];
printf("k=%d, Sopt[k] =  %lf\n",k, Sopt[k]);
k++;
}
// plot with DISLIN
 winsiz (600, 600);
 page (2600, 2600);
 sclmod ("full");
 scrmod ("revers");
 metafl ("xwin");

 disini();
 pagera();
 texmod ("on");
 axspos(450,1800);
 axslen(1800,1500);
 name("t","x");
 name("x(t)","y");
 graf(0.,max_steps,0.,0.2,-0.2, 0.5, -0.2,0.1);
 height(50);
//-----------------------------------------------------

rk43(h,max_steps,t,x,y,Sopt[0]);
nn=j;
color("red");
curve(T,X,nn); 
//-----------------------------------------------------
rk43(h,max_steps,t,x,y,Sopt[1]);
nn=j;
color("blue");
curve(T,X,nn); 
//-----------------------------------------------------
rk43(h,max_steps,t,x,y,Sopt[2]);
nn=j;
color("green");
curve(T,X,nn); 
//-----------------------------------------------------

  endgrf();
  disfin ();
}